const requireComponent = require.context('./', true, /\.vue$/)

const install = (Vue) => {
    if(install.installed) return
    install.installed

    requireComponent.keys().forEach(fileName => {
        // 第i个组件
        const config = requireComponent(fileName)
        // 组件名
        const componentName = config.default.name

        Vue.component(componentName, config.default || config)
    });

    // 聚焦
    Vue.directive('focus', {
        inserted: function(el){
            el.focus()
        }
    })

    // 防抖
    Vue.directive('debounce', {
        bind: function (el, binding) {
            try {
                let fn, event = "click", time = 500;
                if (typeof binding.value == 'function') {
                    fn = binding.value
                } else {
                    [fn, event = "click", time = 500] = binding.value
                }
                let timer;
                el.addEventListener(event, () => {
                    timer && clearTimeout(timer)
                    timer = setTimeout(() => fn(), time)
                })
            } catch (e) {
                console.log(e)
            }
        }
    })

    // 节流
    Vue.directive('throttle', {
        bind: function (el, binding) {
            let fn, event = "click", time = 1500;
            if (typeof binding.value == 'function') {
                fn = binding.value
            } else {
                [fn, event = "click", time = 1500] = binding.value
            }
            /**
             * el.preTime 记录上次触发事件，
             * 每次触发比较nowTime（当前时间） 和 el.preTime 的差是否大于指定的时间段！
             */
            el.addEventListener(event, () => {
                const nowTime = new Date().getTime()
                if (!el.preTime || nowTime - el.preTime > time) {
                    el.preTime = nowTime
                    fn();
                }
            })
        }
    })

    // 添加水印
    Vue.directive('waterMarker', {
        bind: function (el, binding) {
            addWaterMarker(binding.value.text, el, binding.value.font, binding.value.textColor)
        },
    })
    function addWaterMarker(str, parentNode, font, textColor) {
        // 水印文字，父元素，字体，文字颜色
        var can = document.createElement('canvas')
        parentNode.appendChild(can)
        can.width = 200
        can.height = 150
        can.style.display = 'none'
        var cans = can.getContext('2d')
        cans.rotate((-20 * Math.PI) / 180)
        cans.font = font || '16px Microsoft JhengHei'
        cans.fillStyle = textColor || 'rgba(180, 180, 180, 0.3)'
        cans.textAlign = 'left'
        cans.textBaseline = 'Middle'
        cans.fillText(str, can.width / 10, can.height / 2)
        parentNode.style.backgroundImage = 'url(' + can.toDataURL('image/png') + ')'
    }

    // 拖拽
    Vue.directive('drag', {
        inserted: function (el) {
        el.style.cursor = 'move'
        el.onmousedown = function (e) {
            let disx = e.pageX - el.offsetLeft
            let disy = e.pageY - el.offsetTop
            document.onmousemove = function (e) {
                let x = e.pageX - disx
                let y = e.pageY - disy
                let maxX = document.body.clientWidth - parseInt(window.getComputedStyle(el).width)
                let maxY = document.body.clientHeight - parseInt(window.getComputedStyle(el).height)
                if (x < 0) {
                    x = 0
                } else if (x > maxX) {
                    x = maxX
                }
        
                if (y < 0) {
                    y = 0
                } else if (y > maxY) {
                    y = maxY
                }
        
                el.style.left = x + 'px'
                el.style.top = y + 'px'
            }
            document.onmouseup = function () {
                document.onmousemove = document.onmouseup = null
            }
        }
        },
    })

    // 长按
    Vue.directive('longpress', {
        bind: function (el, binding) {
            if (typeof binding.value !== 'function') {
                throw 'callback must be a function'
            }
            // 定义变量
            let pressTimer = null
            let time = binding.value.time || 2000
            // 创建计时器（ 2秒后执行函数 ）
            let start = (e) => {
                if (e.type === 'click' && e.button !== 0) {
                    return
                }
                if (pressTimer === null) {
                    pressTimer = setTimeout(() => {
                        handler()
                    }, time)
                }
            }
            // 取消计时器
            let cancel = () => {
                if (pressTimer !== null) {
                    clearTimeout(pressTimer)
                    pressTimer = null
                }
            }
            // 运行函数
            const handler = (e) => {
                binding.value(e)
            }
            // 添加事件监听器
            el.addEventListener('mousedown', start)
            el.addEventListener('touchstart', start)
            // 取消计时器
            el.addEventListener('click', cancel)
            el.addEventListener('mouseout', cancel)
            el.addEventListener('touchend', cancel)
            el.addEventListener('touchcancel', cancel)
        },
        // 当传进来的值更新的时候触发
        componentUpdated(el, { value }) {
            el.$value = value
        },
        // 指令与元素解绑的时候，移除事件绑定
        unbind(el) {
            el.removeEventListener('click', el.handler)
        },
    })

}

// 环境检测
if(typeof window !== 'undefined' && window.Vue){
    install(window.Vue)
}

export default {
    install
}